/*  Copyright (C) 2017 Nicholas C. Strauss (strauss@positive-internet.com)
    This file is part of ORBTIMES.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __EVALUATEH__
#define __EVALUATEH__

#include <stdint.h>

enum funcall_e {
  i_sunrise,
  i_sunset,
  i_azimuth,
  i_altitude,
  i_nighttime,
  i_daytime,
  i_moonrise,
  i_moonset,
  i_moon_azimuth,
  i_moon_altitude,
  i_moon_nighttime,
  i_moon_daytime,
  //mercury
  i_mercuryrise,
  i_mercuryset,
  i_mercury_azimuth,
  i_mercury_altitude,
  i_mercury_nighttime,
  i_mercury_daytime,
  //venus
  i_venusrise,
  i_venusset,
  i_venus_azimuth,
  i_venus_altitude,
  i_venus_nighttime,
  i_venus_daytime,
  //mars
  i_marsrise,
  i_marsset,
  i_mars_azimuth,
  i_mars_altitude,
  i_mars_nighttime,
  i_mars_daytime,
  //jupiter
  i_jupiterrise,
  i_jupiterset,
  i_jupiter_azimuth,
  i_jupiter_altitude,
  i_jupiter_nighttime,
  i_jupiter_daytime,
  //saturn
  i_saturnrise,
  i_saturnset,
  i_saturn_azimuth,
  i_saturn_altitude,
  i_saturn_nighttime,
  i_saturn_daytime,
  //uranus
  i_uranusrise,
  i_uranusset,
  i_uranus_azimuth,
  i_uranus_altitude,
  i_uranus_nighttime,
  i_uranus_daytime,
  //neptune
  i_neptunerise,
  i_neptuneset,
  i_neptune_azimuth,
  i_neptune_altitude,
  i_neptune_nighttime,
  i_neptune_daytime,
  i_now,
  i_month,
  i_dow,
  i_year,
  i_jan,
  i_feb,
  i_mar,
  i_apr,
  i_may,
  i_jun,
  i_jul,
  i_aug,
  i_sep,
  i_oct,
  i_nov,
  i_dec,                      
  i_maxFuncalls,
};

typedef struct {
  uint16_t            itype;
  char                str[32];
  double              (*handler)();
} funcall_t;

void initialize_cvalues();
double evaluate_cvalue(char *cvalue, bool *truth);
double evaluate(SExpression *e);

// handlers
double handle_isnighttime();
double handle_isdaytime();
double handle_sunrise();
double handle_sunset();
double handle_azimuth();
double handle_altitude();
double handle_moonrise();
double handle_moonset();
double handle_moon_azimuth();
double handle_moon_altitude();
double handle_moon_isnighttime();
double handle_moon_isdaytime();
// mercury
double handle_mercuryrise();
double handle_mercuryset();
double handle_mercury_azimuth();
double handle_mercury_altitude();
double handle_mercury_isnighttime();
double handle_mercury_isdaytime();
// venus
double handle_venusrise();
double handle_venusset();
double handle_venus_azimuth();
double handle_venus_altitude();
double handle_venus_isnighttime();
double handle_venus_isdaytime();
// mars
double handle_marsrise();
double handle_marsset();
double handle_mars_azimuth();
double handle_mars_altitude();
double handle_mars_isnighttime();
double handle_mars_isdaytime();
// jupiter
double handle_jupiterrise();
double handle_jupiterset();
double handle_jupiter_azimuth();
double handle_jupiter_altitude();
double handle_jupiter_isnighttime();
double handle_jupiter_isdaytime();
// saturn
double handle_saturnrise();
double handle_saturnset();
double handle_saturn_azimuth();
double handle_saturn_altitude();
double handle_saturn_isnighttime();
double handle_saturn_isdaytime();
// uranus
double handle_uranusrise();
double handle_uranusset();
double handle_uranus_azimuth();
double handle_uranus_altitude();
double handle_uranus_isnighttime();
double handle_uranus_isdaytime();
// neptune
double handle_neptunerise();
double handle_neptuneset();
double handle_neptune_azimuth();
double handle_neptune_altitude();
double handle_neptune_isnighttime();
double handle_neptune_isdaytime();
// end planets
double handle_now();
double handle_month();
double handle_dow();
double handle_year();
double handle_jan();
double handle_feb();
double handle_mar();
double handle_apr();
double handle_may();
double handle_jun();
double handle_jul();
double handle_aug();
double handle_sep();
double handle_oct();
double handle_nov();
double handle_dec();

#endif
