/*  Copyright (C) 2017 Nicholas C. Strauss (strauss@positive-internet.com)
    This file is part of ORBTIMES.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Based on formulae in Peter Duffett's book
   "Astronomy with your Personal Computer."
*/
//#include <Limits.h>
//#include <Types.h>
#include <math.h>
//#include "alerts.h"
//#include <SANE.h>
//#include "angle.h"
//#include <OSUtils.h>
//#include <stdio.h>
//#include <string.h>
//#include <strings.h>
//#include <TextUtils.h>
#include "startime.h"
#include "sun.h"
#include "moon2.h"
////#include "macmac.h"

#define DEG2RAD(x) 1.745329252e-2L * (x)
#define RAD2DEG(x) 5.729577951e1L * (x)
#define SQR(x) (x)*(x)
#define REDUCE_DEG(x) (360*(x - rint(x)))

const extended TP = 2 * M_PI;

// Apparent Geocentric Ecliptic Longitude, Latitude, Horizontal parallax, 
// To get true, add in nutation.
// Reference: Peter Duffett-Smith's 6000 MOON
void calculate_moon(extended dj, extended ut, extended &mm, extended &bm, extended &pm,
				extended &distance_to_moon, extended &phase, 
				extended &solar_longitude, extended &solar_radius, Boolean computeSolarArguments)
{
	extended		t, t2;
	extended 		m1, m2, m3, m4, m5, m6;
	extended        q; 
	// mean longitude, sun mean anomaly, moon mean anomaly, mean elongation, 
	// mean distance from ascending node, longitude of ascending node
	extended        ml, ms, md, me, mf, na;
	extended        a, b, c, s1, s2, s3, s4, e, e2;
	extended        l, g, w1, w2;
    //extended        mm, bm, pm;
   // extended        sr, rr, d2, d3;
    extended        d2, d3;
    extended        d, i, dp, d0;
    extended 		am;
    
t = (dj/36525.0) + (ut/8.766e5);               // dj is universal time and julian date
t2 = SQR(t);
m1 = 2.732158213e1;
m2 = 3.652596407e2;
m3 = 2.755455094e1;
m4 = 2.953058868e1;
m5 = 2.721222039e1;
m6 = 6.798363307e3;
q = dj + (ut/24.0);
m1 = q/m1;
m2 = q/m2;
m3 = q/m3;
m4 = q/m4;
m5 = q/m5;
m6 = q/m6;

m1 = REDUCE_DEG(m1);
m2 = REDUCE_DEG(m2);
m3 = REDUCE_DEG(m3);
m4 = REDUCE_DEG(m4);
m5 = REDUCE_DEG(m5);
m6 = REDUCE_DEG(m6);

ml = 2.70434164e2 + m1 - (1.133e-3 -1.90e-6 * t)*t2;
ms = 3.58475833e2 + m2 - (1.500e-4 +3.30e-6 * t)*t2;
md = 2.96104608e2 + m3 + (9.192e-3 +1.44e-5 * t)*t2;
me = 3.50737486e2 + m4 - (1.436e-3 -1.90e-6 * t)*t2;
mf = 11.250889    + m5 - (3.211e-3 + 3e-7   * t)*t2;
na = 2.59183275e2 - m6 + (2.078e-3 + 2.2e-6 * t)*t2;

a = DEG2RAD(51.2 + 20.2*t);
s1 = sin(a);
s2 = sin(DEG2RAD(na));
b = 346.56 + (132.87 - 9.1731e-3 * t) * t;
s3 = 3.964e-3 * sin(DEG2RAD(b));
c = DEG2RAD(na + 275.05 - 2.3 * t);
s4 = sin( c );
ml += 2.33e-4 * s1 + s3 + 1.964e-3 * s2;
ms += -1.778e-3 * s1;
md += 8.17e-4 * s1 + s3 + 2.541e-3 * s2;
mf += s3 - 2.4691e-2 * s2 - 4.328e-3 * s4;
me += 2.011e-3 * s1 + s3 + 1.964e-3 * s2;
e = 1 - (2.495e-3 + 7.52e-6 * t) * t;
e2 = SQR(e);
ml = DEG2RAD(ml);
ms = DEG2RAD(ms); 
na = DEG2RAD(na);  
me = DEG2RAD(me);  
mf = DEG2RAD(mf);  
md = DEG2RAD(md);  

l = 6.28875 * sin(md) + 1.274018 * sin(2 * me - md);
l += 6.58309e-1 * sin(2*me) + 2.13616e-1 * sin(2*md);
l +=-e*1.85596e-1 * sin(ms) - 1.14336e-1 * sin(2*mf);
l += 5.8793e-2 * sin(2*(me-md));
l += 5.7212e-2 * e * sin(2*me-ms-md) + 5.332e-2 * sin(2*me + md);
l += 4.5874e-2 * e * sin(2 * me-ms) + 4.1024e-2*e*sin(md-ms);
l +=-3.4718e-2 * sin(me) - e*3.0465e-2 * sin(ms+md);
l += 1.5326e-2 * sin(2*(me-mf)) -1.2528e-2 * sin(2*mf+md);
l +=-1.098e-2 * sin(2*mf -md) + 1.0674e-2 * sin(4*me-md);
l += 1.0034e-2 * sin(3*md) + 8.548e-3 * sin(4*me-2*md);
l +=-e*7.91e-3 * sin(ms-md+2*me) - e*6.783e-3 * sin(2*me+ms);
l += 5.162e-3 * sin(md-me) + e*5e-3 * sin(ms+me);
l += 3.862e-3 * sin(4*me) + e*4.049e-3 * sin(md-ms+2*me);
l += 3.996e-3 * sin(2*(md+me)) + 3.665e-3 * sin(2*me - 3*md);
l += e*2.695e-3 * sin(2*md-ms) + 2.602e-3 * sin(md-2*(mf+me));
l += e*2.396e-3 * sin(2*(me-md)-ms) - 2.349e-3 * sin(md+me); 
l += e2*2.249e-3 * sin(2*(me-ms)) - e*2.125e-3 * sin(2*md+ms); 
l +=-e2*2.079e-3 * sin(2*ms) + e2*2.059e-3 * sin(2*(me-ms)-md); 
l += -1.773e-3 * sin(md + 2*(me-mf)) - 1.595e-3 * sin(2*(mf+me));
l += e*1.22e-3 * sin(4*me-ms-md) -1.11e-3 * sin(2*(md+mf));
l += 8.92e-4 * sin(md-3*me) - e*8.11e-4 * sin(ms+md+2*me);
l += e * 7.61e-4 * sin(4*me-ms-2*md);
l += e2 * 7.04e-4 * sin(md - 2*(ms+me));
l += e * 6.93e-4 * sin(ms-2*(md-me));
l += e * 5.98e-4 * sin(2*(me-mf)-ms);
l += 5.5e-4 * sin(md+4*me) + 5.38e-4 * sin(4*md);
l += e*5.21e-4 * sin(4*me-ms) + 4.86e-4 * sin(2*md-me);
l += e2*7.17e-4 * sin(md-2*ms);
// reduce modulo 2pi
mm = ml + DEG2RAD(l);
mm = fmod(mm, TP);    // moons ecliptic longitude

g = 5.128189 * sin(mf) + 2.80606e-1 * sin(md+mf);
g += 2.77693e-1 * sin(md-mf) + 1.73238e-1 * sin(2*me-mf);
g += 5.54130e-2 * sin(2*me+mf-md) + 4.6272e-2 * sin(2*me-mf-md);
g += 3.25730e-2 * sin(2*me+mf) + 1.7198e-2 * sin(2*md+mf);
g += 9.26700e-3 * sin(2*me+md-mf) + 8.823e-3 * sin(2*md-mf);
g += e*8.24700e-3 * sin(2*me-ms-mf) + 4.323e-3 * sin(2*(me-md)-mf);
g += 4.200e-3 * sin(2*me+mf+md) + e*3.372e-3 * sin(mf-ms-2*me);
g += e * 2.472e-3 * sin(2*me+mf-ms-md);
g += e * 2.222e-3 * sin(2*me+mf-ms);
g += e * 2.072e-3 * sin(2*me-mf-ms-md);
g += e * 1.877e-3 * sin(mf-ms+md) + 1.828e-3*sin(4*me-mf-md);
g +=-e * 1.803e-3 * sin(mf+ms) - 1.75e-3*sin(3*mf);
g += e * 1.570e-3 * sin(md-ms-mf) - 1.487e-3*sin(mf+me);
g +=-e * 1.481e-3 * sin(mf+ms+md) + e*1.417e-3 * sin(mf-ms-md); 
g += e * 1.350e-3 * sin(mf-ms) + 1.33e-3 * sin(mf-me);
g += 1.106e-3 * sin(mf+3*md) + 1.02e-3 * sin(4*me-mf);
g += 8.33e-4 * sin(mf+4*me-md) + 7.81e-4 * sin(md-3*mf);
g += 6.7e-4 * sin(mf+4*me-2*md) + 6.06e-4 * sin(2*me - 3*mf);
g += 5.97e-4 * sin(2*(me+md)-mf);
g += e * 4.92e-4 * sin(2*me+md-ms-mf) + 4.5e-4 * sin(2*(md-me)-mf);
g += 4.39e-4 * sin(3*md-mf) + 4.23e-4 * sin(mf+2*(me+md));
g += 4.22e-4 * sin(2*me-mf-3*md)-e*3.67e-4 * sin(ms+mf+2*me-md);
g +=-e*3.53e-4 * sin(ms+mf+2*me) + 3.31e-4 * sin(mf+4*me);
g += e*3.17e-4 * sin(2*me+mf-ms+md);
g += e2 * 3.06e-4 * sin(2*(me-ms)-mf) -2.83e-4 * sin(md+3*mf);
w1 = 4.664e-4*cos(na);
w2 = 7.54e-5 *cos(c);
bm = DEG2RAD(g) * (1.0 - w1 - w2);   // moons ecliptic latitude

pm = 9.50724e-1 + 5.1818e-2 * cos(md) + 9.531e-3 * cos(2*me-md);
pm += 7.843e-3 * cos(2*me) + 2.824e-3 * cos(2*md);
pm += 8.57e-4 * cos(2*me+md) + e*5.33e-4 * cos(2*me-ms);
pm += e*4.01e-4 * cos(2*me-md-ms);
pm += e*3.2e-4 * cos(md-ms) -2.71e-4 * cos(me);
pm +=-e*2.64e-4 * cos(ms+md) -1.98e-4 * cos(2*mf-md);
pm += 1.73e-4 * cos(3*md) + 1.67e-4 * cos(4*me-md);
pm +=-e*1.11e-4 * cos(ms) + 1.03e-4 * cos(4*me-2*md);
pm +=-8.4e-5 * cos(2*md-2*me) -e*8.3e-5 * cos(2*me+ms);
pm += 7.9e-5 * cos(2*me+2*md) +7.2e-5 * cos(4*me);
pm += e*6.4e-5 * cos(2*me-ms+md) -e*6.3e-5 * cos(2*me+ms-md);
pm += e*4.1e-5 * cos(ms+me) + e*3.5e-5 * cos(2*md-ms);
pm +=-3.3e-5 * cos(3*md-2*me) -3e-5 * cos(md+me);
pm +=-2.9e-5 * cos(2*(mf-me)) -e*2.9e-5 * cos(2*md+ms);
pm += e2*2.6e-5 * cos(2*(me-ms)) -2.3e-5 * cos(2*(mf-me)+md);
pm += e*1.9e-5 * cos(4*me-ms-md);
pm = DEG2RAD(pm);                  // moons horizontal parallax

distance_to_moon = 6378.14/sin(pm);  // in kilometers 

if (computeSolarArguments)
sun_position(dj, ut, solar_longitude, solar_radius, d2, d3, am); 

 d = acos(cos(mm - DEG2RAD(solar_longitude)) * cos(bm));
 i = 1.468e-1 * sin(d) * (1.0 - 5.49e-2 * sin(md));
 i /= (1 - 1.67e-2 * sin(ms));
 i = M_PI - d - DEG2RAD(i);
 phase = (1+cos(i))/2; 
 bm = RAD2DEG(bm);
 mm = RAD2DEG(mm);
 // this is for apparent position
 nutation(dj, dp, d0);
 mm += dp;
 pm = RAD2DEG(pm);
}
