/*  Copyright (C) 2017 Nicholas C. Strauss (strauss@positive-internet.com)
    This file is part of ORBTIMES.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
//#include "macmac.h"
//#include "menu.h"
//#include <Picker.h>	
//#include <Scrap.h>
//#include <Resources.h>
//#include <SegLoad.h>
#include <math.h>
#include <stdio.h>
#include <strings.h>
#include "startime.h"
//#include "application.h"

//#include <OSUtils.h>
//#include <TextUtils.h>
//#include "util.h"
#include "angle.h"
//#include <SANE.h>
//#include "alerts.h"
//#include "preferences.h"
#include "sun.h"
#include "planets.h"
#include "equationOfTime.h"
#define MAIN extern
#include "globals.h"

#define FNU(x) (x)-rint(x/6.283185308) * 6.283185308

#define cos(x) cosl(x)
#define sin(x) sinl(x)
#define atan(x) atanl(x)
#define tan(x) tanl(x)

// Perturbations based on Gaillot, Annales de l'Observatoire 
// de Paris, Memoires, 1904, 1913, 1910
void compute_planet(extended dj, extended ut, short planetNumber);
void mercury_perturbation(int ip, extended t, extended ms, extended &qa,  extended &qb,
	  				extended &qc,  extended &qd,  extended &qe,  extended &qf,  extended &qg);
void venus_perturbation(int ip, extended t, extended ms, extended &qa,  extended &qb,
	  				extended &qc,  extended &qd,  extended &qe,  extended &qf,  extended &qg);
void mars_perturbation(int ip, extended t, extended ms, extended &qa,  extended &qb,
	  				extended &qc,  extended &qd,  extended &qe,  extended &qf,  extended &qg);
void jupiter_perturbation(int ip, extended t, extended ms, extended &qa,  extended &qb,
	  				extended &qc,  extended &qd,  extended &qe,  extended &qf,  extended &qg);
void saturn_perturbation(int ip, extended t, extended ms, extended &qa,  extended &qb,
	  				extended &qc,  extended &qd,  extended &qe,  extended &qf,  extended &qg);
void uranus_perturbation(int ip, extended t, extended ms, extended &qa,  extended &qb,
	  				extended &qc,  extended &qd,  extended &qe,  extended &qf,  extended &qg);
void neptune_perturbation(int ip, extended t, extended ms, extended &qa,  extended &qb,
	  				extended &qc,  extended &qd,  extended &qe,  extended &qf,  extended &qg);

#define SQR(x) (x)*(x)
const extended TP = 2 * M_PI;
struct elements_coeff {
	extended 	a0, a1, a2, a3;
};

// mercury
struct planet_elements_coeff mercury_element_c={
	{178.179078, 415.2057519, 3.011e-4, 0},
 	{75.899697, 1.5554889, 2.947e-4, 0},
 	{2.0561421e-1, 2.046e-5, -3e-8,0 },
 	{7.002881, 1.8608e-3, -1.83e-5, 0},
 	{47.145944, 1.1852083, 1.739e-4, 0},
 	3.870986e-1, 6.74, -0.42,
 	mercury_perturbation
 	};
// venus
struct planet_elements_coeff venus_element_c={
	{342.767053, 162.5533664, 3.097e-4, 0},
 	{130.163833, 1.4080361, -9.764e-4, 0},
 	{6.82069e-3, -4.774e-5, 9.1e-8 ,0 },
 	{3.393631, 1.0058e-3, -1e-6, 0},
 	{75.779647, 8.9985e-1, 4.1e-4, 0},
 	7.233316e-1, 16.92, -4.4,
 	venus_perturbation
 	};
// mars
struct planet_elements_coeff mars_element_c={
	{293.737334, 53.17137642, 3.107e-4, 0},
 	{3.34218203e2, 1.8407584, 1.299e-4, -1.19e-6},
 	{9.33129e-2, 9.2064e-5, -7.7e-8,0 },
 	{1.850333, -6.75e-4, 1.26e-5, 0},
 	{48.786442, 7.709917e-1, -1.4e-6, -5.33e-6},
 	1.5236883, 9.36, -1.52,
 	mars_perturbation
 	};
// jupiter
struct planet_elements_coeff jupiter_element_c={
	{238.049257, 8.434172183, 3.347e-4, -1.65e-6},
 	{1.2720972e1, 1.6099617, 1.05627e-3, -3.43e-6},
 	{4.833475e-2, 1.6418e-4, -4.676e-7, -1.7e-9 },
 	{1.308736, -5.6961e-3, 3.9e-6, 0},
 	{99.443414, 1.01053, 3.5222e-4, -8.51e-6},
 	5.202561, 196.74, -9.4,
 	jupiter_perturbation
 	};
// saturn
struct planet_elements_coeff saturn_element_c={
	{266.564377, 3.398638567, 3.245e-4, -5.8e-6},
 	{9.1098214e1, 1.9584158, 8.2636e-4, 4.61e-6},
 	{5.589232e-2, -3.455e-4, -7.28e-7, 7.4e-10 },
 	{2.492519, -3.9189e-3, -1.549e-5, 4e-8},
 	{112.790414, 8.731951e-1, -1.5218e-4, -5.31e-6},
 	9.554747, 165.6, -8.88,
 	saturn_perturbation
 	};
// uranus 

struct planet_elements_coeff uranus_element_c={
	{244.19747, 1.194065406, 3.16e-4, -6e-7},
 	{1.71548692e2, 1.4844328, 2.372e-4, -6.1e-7},
 	{4.63444e-2, -2.658e-5, 7.7e-8, 0 },
 	{7.72464e-1, 6.253e-4, 3.95e-5, 0},
 	{73.477111, 4.986678e-1, 1.3117e-3, 0},
 	19.21814, 65.8, -7.19,
 	uranus_perturbation
 	};
 // neptune
struct planet_elements_coeff neptune_element_c={
	{84.457994, 6.107942056e-1, 3.205e-4, -6e-7},
 	{4.6727364e1, 1.4245744, 3.9082e-4, -6.05e-7},
 	{8.99704e-3, 6.33e-6, -2e-9, 0 },
 	{1.779242, -9.5436e-3, -9.1e-6, 0},
 	{130.681389, 1.098935, 2.4987e-4, -4.718e-6},
 	30.10957, 62.2, -6.87,
    neptune_perturbation
 	};

struct planet_elements_coeff *all_planet_elements_coeff[]={
	&mercury_element_c, &venus_element_c, &mars_element_c, &jupiter_element_c, 
	&saturn_element_c, &uranus_element_c, &neptune_element_c };
	
struct planet_elements 		mercury_element;
struct planet_elements 		venus_element;
struct planet_elements 		mars_element;
struct planet_elements 		jupiter_element;
struct planet_elements 		saturn_element;
struct planet_elements 		uranus_element;
struct planet_elements 		neptune_element;

struct planet_elements *all_planet_elements[]={
	&mercury_element, &venus_element, &mars_element, &jupiter_element, 
	&saturn_element, &uranus_element, &neptune_element };

static extended planet_anomaly[7];

/* results in degrees
   fixed lots of bugs 12/29/00 
 */
void compute_planet_position(short planetNumber, extended dj, extended tz, extended ds,
	extended gl, extended gp, extended solar_longitude,
	extended &ecliptic_lat, extended &ecliptic_lon, 
	extended &ra, extended &dec, extended &value_min, extended &value_max)
{
  extended			l0, s0, p0, v0;
  extended			dp, d0;
  extended          a, b, c, p, q, rh;

  compute_planet(dj, 0.0, planetNumber, l0, s0, p0, ecliptic_lon, ecliptic_lat, v0, rh);
  nutation(dj, dp, d0);
  ecliptic_lon += DEG2RAD(dp);   // units are degrees.
  a = DEG2RAD(solar_longitude) + M_PI - ecliptic_lon;// what lg?
  b = cos(a);
  c = sin(a);
  ecliptic_lon += -(9.9387e-5*b/cos(ecliptic_lat));
  ecliptic_lat += -(9.9387e-5*c*sin(ecliptic_lat));
  ecliptic_lon = RAD2DEG(ecliptic_lon);
  ecliptic_lat = RAD2DEG(ecliptic_lat);
  p = ecliptic_lon;
  q = ecliptic_lat;
  geocentric_ecliptic_latlon_to_radec(-1, ra, dec, p, q, dj);
  // geocentric_ecliptic_latlon_to_radec(-1, dec, ra, ecliptic_lat, ecliptic_lon, dj);
  //if (ra > 360.0) ra -= 180.0;
  // if (ra < -180.0) ra += 180.0;
  ra = 24.0/360.0 * ra;
  if (ra < 0) ra += 24.0;
 //if (ra > 10.0) ra -= 24.0;
 //if (ra < -10.0) ra += 24.0;
  value_min = MIN(value_min, ra);
  value_max = MAX(value_max, ra);
}


/* void compute_parallax(extended dj, extended ut, extended tz, extended ds, */
/* 		      extended gl, extended gp, extended &x, extended &y, extended &p, extended &q) */
/* { */
/* } */
  
void compute_planet_az_el(short planetNumber, extended dj, extended ut, extended tz, extended ds,
			  extended gl, extended gp, extended solar_lon,
			  extended &ecliptic_lat, extended &ecliptic_lon,
			  extended &ra, extended &dec, extended &value_min, extended &value_max, extended &az, extended &el)
{
  double         lha;// ut = (dj - floor(dj)-0.5)*24.0;

  compute_planet_position(planetNumber, dj, tz, ds, gl, gp, solar_lon, ecliptic_lat, ecliptic_lon, ra, dec, value_min, value_max);
  lha = local_mean_sidereal_time(dj,ut,gl) - ra;
  observer_coord(lha*15, dec, gp, az, el);
  jdays_planets_azimuth[planetNumber]=az;
  jdays_planets_alitude[planetNumber]=el;
}

/*
   utu = time of rising,
   au = azimuth of rising 
   utd = time of setting 
   ad = azimuth of setting
*/
void planet_calculate_rise(short planetNumber, extended &utu, extended &utd, 
	extended dj, extended tz, extended ds,
	extended gl, extended gp, int &never_rises, int &never_sets,
	extended &au, extended &ad)
{
  extended			utv, l0, s0, p0, ep, bp, v0, cbp;
  extended			dp, d0;
  extended          lg, a, b, c, p, q;
  extended			rh, di, lu, ld;
  extended     		solar_longitude, solar_radius, d2, d3, am;
  
#if 0
  never_rises = true;
  return;
#endif

  sun_position(dj, 0.0, solar_longitude, solar_radius, d2, d3, am); 
  compute_planet(dj, 0.0, planetNumber, l0, s0, p0, ep, bp, v0, rh);
  nutation(dj, dp, d0);
  ep += DEG2RAD(dp);   // units are degrees.
  a = DEG2RAD(solar_longitude) + M_PI - ep;// what lg?
  b = cos(a);
  c = sin(a);
  cbp = cos(bp);
  if (fabs(cbp) < 1e-5) SysBreakStr("planet_calculate_rise");
  ep += -(9.9387e-5*b/cbp);
  bp += -(9.9387e-5*c*sin(bp));
  ep = RAD2DEG(ep);
  bp = RAD2DEG(bp);
  geocentric_ecliptic_latlon_to_radec(-1, p, q, ep, bp, dj);
  if (fabs(rh) < 1e-5) SysBreakStr("planet_calculate_rise");
  a = (all_planet_elements[planetNumber]->angular_diameter/2.0 - 8.79)/rh;
  di = DEG2RAD((34.0+a/60.0)/60.0);
  
  time_of_rising(p, q, di, gp, lu, ld, ad, au, never_sets, never_rises);
  
  if (never_sets == true || never_rises == true) return;

 //SysDebugStrHMS(lu, "lu");
 //SysDebugStrHMS(ld, "ld");
 time_sidereal_to_universal(utu, ds, tz, gl, dj, lu);
 time_sidereal_to_universal(utd, ds, tz, gl, dj, ld);
#if 0
 a = utu + tz + ds;
 
 if (a > 24.0){
 planet_calculate_rise(planetNumber, utu, utd, dj-1.0, tz, ds, gl, gp, 
 						 never_rises, never_sets, au, ad);
 }
 else 
 if (a < 0.0){
 planet_calculate_rise(planetNumber, utu, utd, dj+1.0, tz, ds, gl, gp, 
 						never_rises, never_sets, au, ad);
 }
#endif
}

// GIVEN EARTH'S POSITION.
void compute_planet(extended dj, extended ut, short planetNumber,
		extended &l0, extended &s0, extended &p0,
		extended &ep, extended &bp, extended &v0, extended &rh)
{
	int 								i, k;
	struct planet_elements				*pe;
	struct planet_elements_coeff		*pec;
	extended 							sr, rr, d2, d3, am; 
	extended                    		ms, re, lg;
	extended 							li = 0.0;
	extended							qa =0.0, qb =0.0, 
										qc =0.0, qd =0.0, 
										qe =0.0, qf =0.0, qg =0.0;
	extended 							t = (dj/36525.)+(ut/8.766e5); 
	extended 			ec, pv, lp, om, lo, so, co, in, sp, y;
	extended			ps, pd, ae, at, ci, rd, ll, l1, l2;  
   // initialize 
   // compute all_planet_elements
   compute_planet_elements(t, *all_planet_elements[MERCURY], mercury_element_c);
   compute_planet_elements(t, *all_planet_elements[VENUS], venus_element_c);
   compute_planet_elements(t, *all_planet_elements[MARS], mars_element_c);
   compute_planet_elements(t, *all_planet_elements[JUPITER], jupiter_element_c);
   compute_planet_elements(t, *all_planet_elements[SATURN], saturn_element_c);
   compute_planet_elements(t, *all_planet_elements[URANUS], uranus_element_c);
   compute_planet_elements(t, *all_planet_elements[NEPTUNE], neptune_element_c);

   // earth coordinates.
   sun_position(dj, ut, sr, rr, d2, d3, am);
   ms = am;       // degrees
   re = rr;
   lg = DEG2RAD(sr) + M_PI;  // radians

   // two pass
   for (k=0;k<2;k++){
   // compute mean anomaly
   for (i=0;i<7;i++){
		pe = all_planet_elements[i]; 
   		planet_anomaly[i] = DEG2RAD( pe->mean_longitude - 
   					pe->perihelion_longitude - li*pe->daily_longitude );
    }

   qa = 0; qb = 0; qc = 0; qd = 0; qe = 0; qf = 0; qg = 0;
   //li = 0;
   pec = all_planet_elements_coeff[planetNumber];
   pec->perturbation(planetNumber, t, ms, qa, qb, qc, qd, qe, qf, qg);
   ec = all_planet_elements[planetNumber]->eccentricity + qd;   // eccentricity
   am = planet_anomaly[planetNumber] + qe;                      // perturbed mean anomaly
   // ae = eccentric anomaly
   // at = true anomaly (degrees)
   anomaly(RAD2DEG(am), ec, ae, at);                            

   pv = (all_planet_elements[planetNumber]->semi_major_axis + qf)
   		*(1 - ec*ec)/(1+ec*cos(DEG2RAD(at)));
   		
   lp = all_planet_elements[planetNumber]->perihelion_longitude;
   lp += at + RAD2DEG(qc-qe);
   lp = DEG2RAD(lp);
   
   om = DEG2RAD(all_planet_elements[planetNumber]->ascending_node_longitude);
   lo = lp - om;
   so = sin(lo);
   co = cos(lo);
   in = DEG2RAD(all_planet_elements[planetNumber]->inclination);
   pv += qb;
   y  = so*cos(in);
   sp = so*sin(in);
   ps = asin(sp)+qg;
   sp = sin(ps);
   pd = atan(y/co)+om+DEG2RAD(qa);
   if (co < 0) pd += M_PI;
   if (pd > TP) pd -= TP;
   
   ci = cos(ps);
   rd = pv*ci;
   ll = pd-lg;
   rh = re*re + pv*pv - 2*re*pv*ci*cos(ll);
   rh = sqrt(rh);
   li = rh*5.775518e-3;
   if (k==0){
   		l0 = pd;
   		s0 = ps;
   		v0 = rh;
   		p0 = pv;
   	}
   }//end on k
   l1 = sin(ll);
   l2 = cos(ll);
   	if (planetNumber == MERCURY || planetNumber == VENUS) //|| planetNumber == MARS)
   		ep = atan(-rd * l1/(re - rd*l2)) + lg + M_PI;
	else
		ep = atan(re * l1/(rd - re*l2)) + pd;
	while (ep < 0) ep += TP;
	while (ep > TP) ep -= TP;
	bp = atan(rd*sp*sin(ep-pd)/(ci*re*l1));
}

void compute_planet_elements(extended t,
	struct planet_elements &pe, struct planet_elements_coeff &pec)
{
	extended 	//t = (dj/36525.)+(ut/8.766e5),
			    *a  = &pec.mean_longitude[0],
				aa, b, c;
				
	aa = a[1]*t;
	b = 360.0*(aa-rint(aa));
	c = a[0]+b+(a[3]*t+a[2])*t*t;
	while (c>360.0) c -= 360.0;
	while (c<0.0) c += 360.0;
	pe.mean_longitude = c;
	pe.daily_longitude = a[1]*9.856263e-3 + (a[2]+a[3])/36525.0;
	
	a =  &pec.perihelion_longitude[0];
	pe.perihelion_longitude = ((a[3]*t+a[2])*t+a[1])*t+a[0];
	
	a =  &pec.eccentricity[0]; 
	pe.eccentricity = ((a[3]*t+a[2])*t+a[1])*t+a[0];
	
	a =  &pec.inclination[0]; 
	pe.inclination = ((a[3]*t+a[2])*t+a[1])*t+a[0];
	 
	a =  &pec.ascending_node_longitude[0];
	pe.ascending_node_longitude =  ((a[3]*t+a[2])*t+a[1])*t+a[0];
	
	pe.semi_major_axis =  pec.semi_major_axis;
	pe.angular_diameter = pec.angular_diameter;
	pe.standard_magnitude =  pec.standard_magnitude; 
}


void mercury_perturbation(int ip, extended t, extended ms, extended &qa,  extended &qb,
	  				extended &qc,  extended &qd,  extended &qe,  extended &qf,  extended &qg)
{
qa  = 2.04e-3 * cos(5*planet_anomaly[VENUS]-2*planet_anomaly[MERCURY]+2.1328e-1);
qa += 1.03e-3 * cos(2*planet_anomaly[VENUS]-planet_anomaly[MERCURY]-2.8046);
qa += 9.10e-4 * cos(2*planet_anomaly[JUPITER]-planet_anomaly[MERCURY]-6.4582e-1);
qa += 7.80e-4 * cos(5*planet_anomaly[VENUS]-3*planet_anomaly[MERCURY]+1.7692e-1);

qb  = 7.525e-6 * cos(2*planet_anomaly[JUPITER]-planet_anomaly[MERCURY]+9.25251e-1);
qb += 6.802e-6 * cos(5*planet_anomaly[VENUS]-3*planet_anomaly[MERCURY]-4.53642);
qb += 5.457e-6 * cos(2*planet_anomaly[VENUS]-2*planet_anomaly[MERCURY]-1.24246);
qb += 3.569e-6 * cos(5*planet_anomaly[VENUS]-planet_anomaly[MERCURY]-1.35699);
}

//    1        2      3      4       5       6        7
// { MERCURY, VENUS, MARS, JUPITER, SATURN, URANUS, NEPTUNE };
void venus_perturbation(int ip, extended t, extended ms, extended &qa,  extended &qb,
	  				extended &qc,  extended &qd,  extended &qe,  extended &qf,  extended &qg)
{
 qc = DEG2RAD(7.7e-4*sin(4.1406+t*2.6227));
 qe = qc;
 qa = 3.13e-3 * cos(2*ms-2*planet_anomaly[VENUS]-2.587);
 qa+= 1.98e-3 * cos(3*ms-3*planet_anomaly[VENUS]+4.4768e-2);
 qa+= 1.36e-3 * cos(ms-planet_anomaly[VENUS]-2.0788);
 qa+= 9.6e-4  * cos(3*ms-2*planet_anomaly[VENUS]-2.3721);
 qa+= 8.2e-4  * cos(planet_anomaly[JUPITER]-planet_anomaly[VENUS]-3.6318);
 
 qb+= 2.2501e-5 * cos(2*ms-2*planet_anomaly[VENUS]-1.01592);
 qb+= 1.9045e-5 * cos(3*ms-3*planet_anomaly[VENUS]+1.61577);
 qb+= 6.887e-6  * cos(planet_anomaly[JUPITER]-planet_anomaly[VENUS]-2.06106);
 qb+= 5.172e-6  * cos(ms-planet_anomaly[VENUS]-5.08065e-1);
 qb+= 3.62e-6   * cos(5*ms-4*planet_anomaly[VENUS]-1.81877);
 qb+= 3.283e-6  * cos(4*ms-4*planet_anomaly[VENUS]+1.10851);
 qb+= 3.074e-6  * cos(2*planet_anomaly[JUPITER]-2*planet_anomaly[VENUS]-9.62846e-1);
 
}
void mars_perturbation(int ip, extended t, extended ms, extended &qa,  extended &qb,
	  				extended &qc,  extended &qd,  extended &qe,  extended &qf,  extended &qg)
{
	extended 	a, sa, ca;
	
	a = 3*planet_anomaly[JUPITER]-8*planet_anomaly[MARS]+4*ms;
	sa = sin(a);
	ca = cos(a);
	qc = -(1.33e-2*sa + 9.33e-3*ca);
	qc = DEG2RAD(qc);
	qe = qc;
	
	qa =  7.05e-3 * cos(planet_anomaly[JUPITER]-planet_anomaly[MARS] - 8.5448e-1);
	qa += 6.07e-3 * cos(2*planet_anomaly[JUPITER]-planet_anomaly[MARS] - 3.2873);
	qa += 4.45e-3 * cos(2*planet_anomaly[JUPITER]-2*planet_anomaly[MARS] - 3.3492);
	qa += 3.88e-3 * cos(ms - 2*planet_anomaly[MARS] + 3.5771e-1);
	qa += 2.38e-3 * cos(ms - planet_anomaly[MARS] + 6.1256e-1);
	qa += 2.04e-3 * cos(2*ms - 3*planet_anomaly[MARS] + 2.7688);
	qa += 1.77e-3 * cos(3*planet_anomaly[MARS] - planet_anomaly[VENUS] - 1.0053);
	qa += 1.36e-3 * cos(2*ms - 4*planet_anomaly[MARS] + 2.6894);
	qa += 1.04e-3 * cos(planet_anomaly[JUPITER] + 3.0748e-1);
	
	qb  = 5.3227e-5 * cos(planet_anomaly[JUPITER]-planet_anomaly[MARS]+7.17864e-1);
	qb += 5.0989e-5 * cos(2*planet_anomaly[JUPITER]-2*planet_anomaly[MARS]-1.77997);
	qb += 3.8278e-5 * cos(2*planet_anomaly[JUPITER]-planet_anomaly[MARS]-1.71617);
	qb += 1.5996e-5 * cos(ms - planet_anomaly[MARS] -9.69618e-1);
	qb += 1.4764e-5 * cos(2*ms - 3*planet_anomaly[MARS] +1.19768);
	qb += 8.966e-6  * cos(planet_anomaly[JUPITER] - 2*planet_anomaly[MARS] +7.61225e-1);
	qb += 7.914e-6  * cos(3*planet_anomaly[3*JUPITER] - 2*planet_anomaly[MARS] -2.43887);
	qb += 7.004e-6  * cos(2*planet_anomaly[3*JUPITER] - 3*planet_anomaly[MARS] -1.79573);
	qb += 6.62e-6   * cos(ms - 2*planet_anomaly[MARS] -1.97575);
	qb += 4.93e-6   * cos(3*planet_anomaly[3*JUPITER] - 3*planet_anomaly[MARS] -1.33069);
	qb += 4.693e-6  * cos(3*ms - 5*planet_anomaly[MARS] +3.32665);
	qb += 4.571e-6  * cos(2*ms - 4*planet_anomaly[MARS] +4.27086);
	qb += 4.409e-6   * cos(3*planet_anomaly[3*JUPITER] - planet_anomaly[MARS] -2.02158);
}

void outer_block(
	extended	t,
	extended	&j1, 
	extended	&j2, 
	extended	&j3, 
	extended	&j4, 
	extended	&j5, 
	extended	&j6){
	j1 = t/5.0+01;
	j2 = FNU(4.14473 + 5.29691e1 * t);
	j3 = FNU(4.641118 + 2.132991e1 * t);
	j4 = FNU(4.250177 + 7.478172 * t);
	j5 = 5.0*j3 - 2.0 * j2;
	j6 = 2.0*j2 - 6.0 * j3 + 3.0 * j4;
}

void jupiter_saturn_block(
	extended j2, extended j3, extended j4, 
	extended j5, extended j6,	
	extended &j7, extended &u1, extended &u2,
	extended &u3, extended &u4, extended &u5, 
	extended &u6, extended &u7, extended &u8, 
	extended &u9, extended &ua, extended &ub, 
	extended &uc, extended &ud, extended &ue, 
	extended &uf, extended &ug, extended &vh)
{
	j7 = j3 - j2;
	u1 = sin(j3);
	u2 = cos(j3);
	u3 = sin(2.0*j3);
	u4 = cos(2.0*j3);
	u5 = sin(j5);
	u6 = cos(j5);
	u7 = sin(2.0*j5);
	u8 = sin(j6);
	u9 = sin(j7);
	ua = cos(j7);
	ub = sin(2.*j7);
	uc = cos(2.*j7);
	ud = sin(3.*j7);
	ue = cos(3.*j7);
	uf = sin(4.*j7);
	ug = cos(4.*j7);
	vh = cos(5.*j7);
}

void jupiter_perturbation(int ip, extended t, extended ms, extended &qa,  extended &qb,
	  				extended &qc,  extended &qd,  extended &qe,  extended &qf,  extended &qg)
{	
	extended 		j1, j2, j3, j4, j5, j6;
	extended 		j7, u1, u2, u3, u4, u5, 
					u6, u7, u8, u9, ua, ub, 
					uc, ud, ue, uf, ug, vh, vk;
	
	// outer block
	outer_block(t, j1, j2, j3, j4, j5, j6);
	// jupiter saturn block
	jupiter_saturn_block(j2, j3, j4, j5, j6,
		j7, u1, u2, u3, u4, u5, u6, u7, u8, u9,
		ua, ub, uc, ud, ue, uf, ug, vh);
		
	qc  = (3.31364e-1 - (1.0281e-2 + 4.692e-3 * j1) * j1) * u5;
	qc += (3.228e-3 - (6.4436e-2 -2.075e-3 * j1) * j1) * u6;
	qc += -(3.083e-3 + (2.75e-4 -4.89e-4 * j1) * j1) * u7;
	qc += 2.472e-3 * u8 + 1.3619e-2 * u9 + 1.8472e-2 * ub;
	qc += 6.717e-3 * ud + 2.775e-3 * uf + 6.417e-3 * ub * u1;
	qc += (7.275e-3 -1.253e-3 * j1) * u9 * u1 + 2.439e-3 * ud * u1;
	qc += -(3.5681e-2 + 1.208e-3 * j1) * u9 * u2 - 3.767e-3 * uc * u1;
	qc += -(3.3839e-2 + 1.125e-3 * j1) * ua * u1 - 4.261e-3 * ub * u2; 
	qc += (1.161e-3 * j1 - 6.333e-3) * ua * u2 + 2.178e-3 * u2;
	qc += -6.675e-3 * uc * u2 - 2.664e-3 * ue * u2 -2.572e-3 * u9 * u3;
	qc += -3.567e-3 * ub * u3 + 2.094e-3 * ua * u4 + 3.342e-3 * uc * u4;
	qc = DEG2RAD(qc);
	
	qd  = (3606 + (130-43 * j1) * j1) * u5 + (1289-580 * j1) * u6;
	qd  += -6764*u9*u1 - 1110*ub*u1 - 224*ud*u1 - 204*u1;
	qd  += (1284 + 116*j1) * ua * u1 + 188*uc*u1;
	qd  += (1460 + 130*j1) * u9 * u2 + 224*ub*u2 - 817*u2;
	qd  += 6074*u2*ua + 992*uc*u2 + 508*ue*u2 + 230*ug*u2;
	qd  += 108*vh*u2 - (956+73*j1)*u9*u3 + 448*ub*u3;
	qd  += 137*ud*u3 + (108*j1-997)*ua*u3 + 480*uc*u3;
	qd  += 148*ue*u3 + (99*j1-956)*u9*u4 + 490*ub*u4;
	qd  += 158*ud*u4 + 179*u4 + (1024+75*j1)*ua*u4;
	qd  += -437*uc*u4 - 132*ue*u4;
	qd  *= 1e-7;
	
	vk  = (7.192e-3 - 3.147e-3 *j1)*u5 - 4.344e-3 *u1;
	vk += (j1*(1.97e-4 * j1 - 6.75e-4) -2.0428e-2)*u6;
	vk += 3.4036e-2 * ua * u1 + (7.269e-3 + 6.72e-4 * j1) * u9 * u1;
	vk +=  5.614e-3 * uc * u1 + 2.964e-3 * ue * u1 + 3.7761e-2 * u9 * u2;
	vk +=  6.158e-3 * ub * u2 - 6.603e-3 * ua * u2 - 5.356e-3  * u9 * u3;
	vk +=  2.722e-3 * ub * u3 + 4.483e-3 * ua * u3;
	vk += -2.642e-3 * uc * u3 + 4.403e-3 * u9 * u4;
	vk += -2.536e-3 * ub * u4 + 5.547e-3 * ua * u4 - 2.689e-3*uc*u4;
	qe = qc - (DEG2RAD(vk)/all_planet_elements[ip]->eccentricity);//PL(IP,4))
	
	qf = 205*ua - 263*u6 + 693*uc + 312*ue + 147*ug + 299*u9*u1;
	qf += 181*uc*u1 + 204*ub*u2 + 111*ud*u2 - 337*ua*u2;
	qf += -111*uc*u2;
	qf *= 1e-6;
}

void saturn_perturbation(int ip, extended t, extended ms, extended &qa,  extended &qb,
	  				extended &qc,  extended &qd,  extended &qe,  extended &qf,  extended &qg)
{
	extended 		j1, j2, j3, j4, j5, j6;
	extended 		j7, u1, u2, u3, u4, u5, 
					u6, u7, u8, u9, ua, ub, 
					uc, ud, ue, uf, ug, vh, vk;
	extended 		ui, uj, uk, ul, vi, un, j8,
					uo, up, uq, ur;
					
	// outer block
	outer_block(t, j1, j2, j3, j4, j5, j6);
	// jupiter saturn block
	jupiter_saturn_block(j2, j3, j4, j5, j6,
		j7, u1, u2, u3, u4, u5, u6, u7, u8, u9,
		ua, ub, uc, ud, ue, uf, ug, vh);
	ui = sin(3.*j3);
	uj = cos(3.*j3);
	uk = sin(4.*j3);
	ul = cos(4.*j3);
	vi = cos(2.*j5);
	un = sin(5.*j7);
	j8 = j4 - j3;
	uo = sin(2.*j8);
	up = cos(2.*j8);
	uq = sin(3.*j8);
	ur = cos(3.*j8);
	
	// mean longitude perturb
	qc = 7.581e-3 * u7 - 7.986e-3 * u8 - 1.48811e-1 * u9;
	qc += -(8.14181e-1 -(1.815e-2 -1.6714e-2 *j1)*j1)*u5;
	qc += -(1.0497e-2 -(1.60906e-1 - 4.1e-3 *j1)*j1)*u6;
	qc += -1.5208e-2 *ud -6.339e-3 *uf -6.244e-3 *u1;
	qc += -1.65e-2 *ub*u1 -4.0786e-2 *ub;
	qc += (8.9310e-3 + 2.728e-3 *j1)*u9*u1 - 5.7750e-3 *ud*u1;
	qc += (8.1344e-2 + 3.206e-3 *j1)*ua*u1 + 1.5019e-2 *uc*u1;
	qc += (8.5581e-2 + 2.494e-3 *j1)*u9*u2 + 1.4394e-2 *uc*u2;
	qc += (2.5328e-2 - 3.117e-3 *j1)*ua*u2 + 6.3190e-3 *ue*u2;
	qc +=  6.3690e-3 * u9 * u3 + 9.156e-3 *ub*u3 + 7.525e-3 *uq*u3;
	qc += -5.2360e-3 * ua * u4 - 7.736e-3 *uc*u4 - 7.528e-3 *ur*u4;
	qc = DEG2RAD(qc);
	
	// eccentricity perturb
	qd = (-7927+(2548 + 91*j1) *j1) *u5;
	qd += (13381 + (1226 -253*j1)*j1)*u6 +(248-121*j1)*u7;
	qd += -(305+91*j1)*vi + 412*ub + 12415*u1;
	qd += (390-617*j1)*u9*u1 + (165-204*j1)*ub*u1;
	qd += 26599*ua*u1 -4687*uc*u1 -1870*ue*u1 -821*ug*u1;
	qd += -377*vh*u1 + 497*up*u1 + (163-611*j1)*u2;
	qd += -12696*u9*u2 -4200*ub*u2 -1503*ud*u2 -619*uf*u2;
	qd += -268*un*u2 -(282+1306*j1)*ua*u2;
	qd += (-86+230*j1)*uc*u2 + 461*uo*u2 -350*u3;
	qd += (2211 -286*j1)*u9*u3 -2208*ub*u3 -568*ud*u3;
	qd += -346*uf*u3 - (2780+222*j1)*ua*u3;
	qd += (2022 +263*j1) * uc*u3 + 248*ue*u3 + 242*uq*u3;
	qd += 467*ur*u3 -490*u4 -(2842 + 279*j1)*u9*u4;
	qd += (128+ 226*j1)*ub*u4 + 224*ud*u4;
	qd += (-1594 +282*j1)*ua*u4 + (2162 -207*j1)*uc*u4;
	qd += 561*ue*u4 + 343*ug*u4 + 469*uq*u4 -242*ur*u4;
	qd += -205*u9*ui + 262*ud*ui +208*ua*uj -271*ue*uj;
	qd += -382*ue*uk - 376*ud*ul;
	qd *= 1e-7;
	
	// longitude of perihelion perturb
	vk = (7.7108e-2 + (7.186e-3 -1.533e-3 *j1)*j1)*u5;
	vk += -7.075e-3 *u9;
	vk += (4.5803e-2 -(1.4766e-2 + 5.36e-4 *j1)*j1)*u6;
	vk += -7.2586e-2 * u2 - 7.5825e-2 *u9*u1 -2.4839e-2 *ub*u1;
	vk += -8.631e-3 *ud*u1 -1.50383e-1 *ua*u2;
	vk += 2.6897e-2 *uc*u2 +1.0053e-2 *ue*u2;
	vk += -(1.3597e-2 + 1.719e-3 *j1)*u9*u3 +1.1981e-2 *ub*u4;
	vk += -(7.742e-3 -1.517e-3 *j1)*ua*u3;
	vk += (1.3586e-2 -1.375e-3 *j1)*uc*u3;
	vk += -(1.3667e-2 -1.239e-3 *j1)*u9*u4;
	vk += (1.4861e-2 + 1.136e-3 *j1)*ua*u4;
	vk += -(1.3064e-2 + 1.628e-3 *j1)*uc*u4;
	qe = qc-(DEG2RAD(vk)/all_planet_elements[ip]->eccentricity); //mean anomaly perturb
	
	// semi-major axis perturb
	qf = 572*u5 -1590*ub*u2 +2933*u6 -647*ud*u2;
	qf += 33629*ua -344*uf*u2 -3081*uc + 2885*ua*u2;
	qf += -1423*ue +(2172+102*j1)*uc*u2 -671*ug;
	qf += 296*ue*u2 -320*vh -267*ub*u3 +1098*u1;
	qf += -778*ua*u3 -2812*u9*u1 +495*uc*u3 +688*ub*u1;
	qf += 250*ue*u3 -393*ud*u1 -856*u9*u4 -228*uf*u1;
	qf += 441*ub*u4 +2138*ua*u1 +296*uc*u4 -999*uc*u1;
	qf += 211*ue*u4 -642*ue*u1 -427*u9*ui -325*ug*u1;
	qf += 398*ud*ui -890*u2 +344*ua*uj +2206*u9*u2;
	qf += -427*ue*uj;
	qf *= 1e-6;
	
	// heliocentric long
	qg = 7.47e-4 *ua*u1 +1.069e-3 *ua*u2 +2.108e-3 *ub*u3;
	qg += 1.261e-3 *uc*u3 +1.236e-3 *ub*u4 -2.075e-3 *uc*u4;
	qg = DEG2RAD(qg);
}

void uranus_neptune_block(extended t, extended j4, extended &j8, extended &j9,
	extended &vj, extended &uu, extended &uv, extended &uw)
{
	j8 = FNU(1.46205 +3.81337*t);
	j9 = 2*j8-j4;
	vj = sin(j9);
	uu = cos(j9);
	uv = sin(2*j9);
	uw = cos(2*j9);
}

void uranus_perturbation(int ip, extended t, extended ms, extended &qa,  extended &qb,
	  				extended &qc,  extended &qd,  extended &qe,  extended &qf,  extended &qg)
{
	extended 		j1, j2, j3, j4, j5, j6;
	extended		j8, j9, vj, uu, uv, uw;		
	extended        ja, jb, jc, vk, a;
	extended		ux, uy, uz, va, vb, vc;
		
	// outer block
	outer_block(t, j1, j2, j3, j4, j5, j6);
	uranus_neptune_block(t, j4, j8, j9, vj, uu, uv, uw);

	ja = j4 - j2;
	jb = j4 - j3;
	jc = j8 - j4;
	
	// mean long perturb
	qc = (8.64319e-1 -1.583e-3*j1) *vj;
	qc += (8.2222e-2 -6.833e-3*j1) *uu +3.6017e-2*uv;
	qc += -3.019e-3*uw +8.122e-3*sin(j6);
	qc = DEG2RAD(qc);
	// long perihelion perturb
	vk = 1.20303e-1*vj +6.197e-3*uv;
	vk += (1.9472e-2 -9.47e-4*j1)*uu;
	// mean anomaly perturb
	qe = qc - (DEG2RAD(vk)/all_planet_elements[ip]->eccentricity);
	// eccentricity perturb
	qd = (163*j1 -3349)*vj + 20981*uu + 1311*uw;
	qd *= 1e-7;
	// semi-major axis perturb
	qf = -3.825e-3*uu;
	// long perturb
	qa = (-3.8581e-2 + (2.031e-3 -1.91e-3*j1) *j1) *cos(j4+jb);
	qa += (1.0122e-2 -9.88e-4*j1) *sin(j4+jb);
	a = (3.4964e-2 - (1.038e-3 -8.68e-4*j1) *j1) * cos(2*j4 +jb);
	qa += a+ 5.594e-3*sin(j4+ 3*jc) -1.4808e-2*sin(ja);
	qa += -5.794e-3*sin(jb) +2.347e-3*cos(jb);
	qa += 9.872e-3*sin(jc) +8.803e-3*sin(2*jc);
	qa += -4.308e-3*sin(3*jc);
	
	ux = sin(jb);
	uy = cos(jb);
	uz = sin(j4);
	va = cos(j4);
	vb = sin(2*j4);
	vc = cos(2*j4);
	//heliocentric long perturb
	qg = (4.58e-4*ux -6.42e-4*uy -5.17e-4* cos(4*jc)) * uz;
	qg += -(3.47e-4*ux + 8.53e-4*uy + 5.17e-4*sin(4*jb))*va;
	qg += 4.03e-4*(cos(2*jc)*vb +sin(2*jc)*vc);
	qg = DEG2RAD(qg);
	
	// radius perturb
	qb = -25948 + 4985*cos(ja) -1230*va +3354*uy;
	qb += 904*cos(2*jc) + 894*(cos(jc)-cos(3*jc));
	qb += (5795*va -1165*uz +1388*vc) *ux;
	qb += (1351*va +5702*uz +1388*vb) *uy;
	qb *= 1e-6;
}

void neptune_perturbation(int ip, extended t, extended ms, extended &qa,  extended &qb,
	  				extended &qc,  extended &qd,  extended &qe,  extended &qf,  extended &qg)
{
	extended 		j1, j2, j3, j4, j5, j6;
	extended		j8, j9, vj, uu, uv, uw;		
	extended		ja, jb, jc, vk, vd, ve;
	extended		vf, vg;
		
	// outer block
	outer_block(t, j1, j2, j3, j4, j5, j6);
	uranus_neptune_block(t, j4, j8, j9, vj, uu, uv, uw);
	
	ja = j8-j2;
	jb = j8-j3;
	jc = j8-j4;
	// mean long perturb
	qc = (1.089e-3*j1 -5.89833e-1)*vj;
	qc += (4.658e-3 *j1 -5.6094e-2) *uu -2.4286e-2*uv;
	qc = DEG2RAD(qc);
	// long perihelion perturb
	vk = 2.4039e-2 *vj -2.5303e-2 * uu + 6.206e-3*uv;
	vk += -5.992e-3*uw;
	// mean anomaly perturb
	qe = qc - (DEG2RAD(vk)/all_planet_elements[ip]->eccentricity);
	// eccentricity
	qd = 4389*vj +1129*uv +4262*uu +1089*uw;
	qd *= 1e-7;
	// semi-major axis
	qf = 8189*uu -817*vj +781*uw;
	qf *= 1e-6;
	
	vd = sin(2*jc);
	ve = cos(2*jc);
	vf = sin(j8);
	vg = cos(j8);
	// longitude
	qa = -9.556e-3*sin(ja) -5.178e-3*sin(jb);
	qa += 2.572e-3*vd -2.972e-3*ve*vf -2.833e-3*vd*vg;
	// heliocentric long
    qg = 3.36e-4*ve*vf +3.64e-4*vd*vg;
    qg = DEG2RAD(qg);
    //radius
    qb = -40596 +4992*cos(ja) + 2744*cos(jb);
    qb += 2044*cos(jc) + 1051*ve;
    qb *= 1e-6;
}







